/* Custom styling for CaDaR documentation */

/* Code blocks */
.highlight {
    border-radius: 4px;
}

/* Admonitions */
.admonition {
    border-radius: 4px;
}

/* Tables */
table {
    border-radius: 4px;
    overflow: hidden;
}

/* Version badge */
.version-badge {
    background: #5e35b1;
    color: white;
    padding: 2px 8px;
    border-radius: 12px;
    font-size: 0.85em;
    font-weight: 500;
}

/* Grid cards */
.grid.cards {
    gap: 1rem;
}

.grid.cards > * {
    transition: transform 0.2s;
}

.grid.cards > *:hover {
    transform: translateY(-2px);
    box-shadow: 0 4px 8px rgba(0,0,0,0.1);
}

/* RTL Support for Arabic text in examples */
.language-arabic {
    direction: rtl;
    text-align: right;
}

/* Syntax highlighting tweaks */
.highlight .nx {
    color: #5e35b1;
}
